function descriptor = mkDescriptor(rgb_img, rgb_reg, rgb_reg_nan, ...
    gray_img, gray_reg, gray_reg_nan, gPb_thin, text_reg_nan, reg_mask, ...
    reg, selection, baseName, options)
% author: dino bellugi
% copyright: blueG soft wear 2010
% generates the descriptor for a given region mask
% inputs:
%   all the images, masks, etc. (will list later)
%   reg = region number
%   selection = logical(21), selects which properties to include
%   baseName = base name for files
%   options = logical(1,3) for print, plot, save
% outputs:
%   descriptor = a vector containing properties of the region
% example:
%   will list this as well soon
% notes:
%   has a bunch of hard-coded parameters (see below)
% naming convention:
%   descName = [baseName '_desc_' region_number]

    %% check command line
    if (nargin < 12 || nargin > 13)
        error(['Usage: descriptor = mkDescriptor(rgb_img, rgb_reg, ', ...
            'rgb_reg_nan, gray_img, gray_reg, gray_reg_nan, gPb_thin, ', ...
            'text_reg_nan, reg_mask, reg, selection, baseName, [options])\n']);
    end
    if (nargin < 3)
        options = true(3,1);
    end
    
    %% options
    messages = options(1);    % print messages
    figures = options(2);     % generate figures
    saves = options(3);       % save to file

    %% selection of properties in descriptor
    % selection(1): Area
    % selection(2): Perimeter
    % selection(3): Eccentricity
    % selection(4): Solidity
    % selection(5): Circularity
    % selection(6): Orientation histograms
    % selection(7): Change of orientation histograms
    % selection(8): Boundary quality histograms
    % selection(9): Color histograms (Red, Green, Blue)
    % selection(10): Super-pixel color histograms (Red, Green, Blue)
    % selection(11): Intensity histogram (Gray)
    % selection(12): Super-pixel intensity histogram (Gray)
    % selection(13): Texton map histogram (Gray)
    % selection(14): Range filter histogram (Gray)
    % selection(15): Super-pixel range filter histogram (Gray)
    % selection(16): Entropy filter histogram (Gray)
    % selection(17): Super-pixel entropy filter histogram (Gray)
    % selection(18): Gray-Level Co-Occurrence Matrix Correlation
    % selection(19): Super-pixel Gray-Level Co-Occurrence Matrix Correlation
    % selection(20): Region entropy
    % selection(21): Super-pixel region entropy

    %% other parameters
    % image size
    [rows cols] = size(reg_mask);

    % number of histogram bins
    obins = 18;     % orientation (max angle = 180)
    kbins = 20;     % change of orientation (bins defined below, caution!)
    qbins = 16;     % boundary quality (always use power of 2 bins)
    cbins = 16;     % color: red, gree, blue (always use power of 2 bins)
    scbins = 16;    % superpixels color (always use power of 2 bins)
    tmbins = 64;    % textons map (best not to bin and leave at full resolution)
    rfbins = 16;    % range filter (always use power of 2 bins)
    srfbins = 16;   % superpixels range filter (always use power of 2 bins)
    efbins = 16;    % entropy filter
    sefbins = 16;   % superpixels entropy filter
    ccbins = 16;    % co-occurrence correlation
    sccbins = 16;   % superpixels co-occurrence correlation
    egbins = 256;   % grayscale bins used for global entropy

    % filter sizes
    ls_size = 21;   % number of points for least squares (must be odd!)
    sp_size = 5;    % superpixel size (must be odd!)
    rf_size = 5;    % range filter
    ef_size = 5;    % entropy filter
    cm_size = 50;   % co-occurrence
    scm_size = round(cm_size/sp_size);  % superpixel co-occurrence

    % min and max levels
    min_rgb = 0;
    max_rgb = 256;
    min_gray = 0;
    max_gray = 256;
    min_text = 0;
    max_text = 64;
    min_angle = -90;
    max_angle = 90;
    min_entr = 1;
    max_entr = 5;
    min_gpb = 0;
    max_gpb = 256;

    %% region properties
    if (messages)
        fprintf('Computing region properties ...\n');
    end

    tic

    % reg_stats holds selected region properties
    % notes: 
    % eccentricity is equivalent to the conventional L/I
    % solidity is the region area / convex hull area ratio
    % solidity is also called irregularity [blott & pye 2008]
    % circularity is the ratio of inscribed to circumscribed circle [riley 1941]
    % maximum inscribed circle of a general shape is a really hard problem,
    % we will use the convex hull instead of the general shape for simplicity

    % get what is already built in matlab's region props
    reg_stats = regionprops(bwlabel(reg_mask), ...
        'Area', 'Perimeter', 'Eccentricity', 'Solidity');

    % get boundary pixel coordinates
    B = bwboundaries(reg_mask, 'noholes');
    boundary = B{1};
    clear B;
    len = size(boundary,1);

    % get max inscribed circle (courtesy of John D'Errico)
    [center1 radius1] = incircle(boundary(:,2), boundary(:,1));

    % get min bounding circle (courtesy of John D'Errico)
    [center2 radius2] = minboundcircle(boundary(:,2), boundary(:,1));

    % note: both the above functions as well as regionpropos call convexhull!
    % remember to modify for efficiency

    % compute circles ratio
    circularity = radius1/radius2;

    toc

    %% histogram orientations
    if (messages)
        fprintf('Computing histogram of orientations ...\n');
    end

    tic

    % compute slope
    slope = zeros(1,len);
    x = zeros(1,ls_size);
    y = zeros(1,ls_size);
    ls_size2 = floor(ls_size/2);

    for i=1:len
        % get ls_size data points centered around i (matlab indices start from 1)
        % note: x and y could be obtained more efficiently from the previous
        % iteration instead of being rebuilt from scratch
        % for ii= -ls_size2:ls_size2
        for ii = 1:ls_size
            x(1,ii) = boundary(mod(i+ii-ls_size2-2,len)+1,2);
            y(1,ii) = boundary(mod(i+ii-ls_size2-2,len)+1,1);
        end

        % fit a line using least squares
        A = [ones(ls_size,1) x'];
        Als = A'*A;
        bls = A'*y';
        lsq = Als\bls;

        % store the slope
        slope(i) = lsq(2);

        % note that the above is equivalent to this:
        % lsq = polyfit(x,y,1);
        % slope(i) = lsq(1);
        % but 10x faster if number of points is not very large.
        % note also that polyfit is a little more robust,
        % it should be chosen when number of points is small.    
    end

    % transform into orientation angle
    orient = atand(slope);

    % set up bins
    binl = (max_angle-min_angle)/obins;
    binv = linspace(min_angle,max_angle,obins+1);
    bint = linspace(min_angle+binl/2,max_angle-binl/2,obins);

    % histogram (normalize by number of pixels on perimeter)
    odis = histc(orient,binv)./len;
    toc

    % plot
    if (figures)
        figure; bar(bint,odis(1:obins));
        title('Orientations'); xlabel('bin'); ylabel('% in bin');
        axis([min_angle max_angle 0 1]); set(gca,'xtick', binv);
    end

    %% histogram change of orientations
    if (messages)
        fprintf('Computing histogram of orientation change ...\n');
    end

    % note: using fwd. diff. of slopes divided by distance as an approximation

    tic

    % compute change in slope
    slope_change = diff([slope slope(1)]);
    subplot
    % compute distances
    distance = ones(1,len+1);
    for i=1:len-1
        if (boundary(i,1) ~= boundary(i+1,1) && boundary(i,2)~=boundary(i+1,2))
            distance(i) = 1.4142;
        end
    end
    if (boundary(len,1) ~= boundary(1,1) && boundary(len,2)~=boundary(1,2))
            distance(len) = 1.4142;
    end

    % divide by distance
    slope_change = slope_change ./ distance(len);

    % set up bins
    % variable binning (kbins = 20, defined above):
    % 2 bins between -5 and -3
    % 4 bins between -3 and -1subplot
    % 8 bins between -1 and 1
    % 4 bins between 1 and 3
    % 2 bins between 3 and 5
    binv = [-5 -4 -3 -2.5 -2 -1.5 -1 -0.75 -0.5 -0.25 0 0.25 0.5 0.75 1 1.5 2 2.5 3 4 5];
    bint = [-4.5 -3.5 -2.75 -2.25 -1.75 -1.25 -0.875 -0.625 -0.375 -0.125 0.125 0.375 0.625 0.875 1.25 1.75 2.25 2.75 3.5 4.5];

    % histogram (normalize by number of pixels on perimeter)
    kdis = histc(slope_change,binv)./len;

    toc

    % plot
    if (figures)
        figure; bar(bint,kdis(1:kbins));
        title('slope changes'); xlabel('bin'); ylabel('% in bin');
        axis([-5 5 0 1]); set(gca,'xtick', binv);
    end
    subplot
    %% histogram boundary quality
    if (messages)
        fprintf('Computing histogram of boundary quality ...\n');
    end

    % note: this part is not exact (but very close) as gPb_thin is slightly 
    % dilated with respect to the boundary, resulting in zero quality values.
    % this is a result of the skeleton construcion (bwmorph in globalPb,m).
    % to avoid going back to gPb adjacency matrix a local 3x3 maximum is used.

    tic

    % thicken gPb with 3x3 max filter
    gPb_thick = ordfilt2(gPb_thin, 9, true(3,3));

    % now get quality using boundary coordinates
    quality = zeros(1,len);
    for i=1:len
        quality(i) = gPb_thick(boundary(i,1), boundary(i,2));
    end

    % set up bins
    binl = (max_gpb-min_gpb)/qbins;
    binv = linspace(min_gpb,max_gpb,qbins+1);
    bint = linspace(min_gpb+binl/2,max_gpb-binl/2,qbins);

    % histogram (normalize by number of pixels in perimeter)
    qdis = histc(quality,binv)./len;
    toc

    % plot
    if (figures)
        figure; bar(bint,qdis(1:qbins));
        title('Boundary quality'); xlabel('bin'); ylabel('% in bin');
        axis([min_gpb max_gpb 0 1]); set(gca,'xtick', binv);
    end

    %% histogram colors and intensity (full resolution)
    if (messages)
        fprintf('Computing pixel histograms of colors ...\n');
    end

    tic

    % reshape image into vectors
    len = rows*cols;
    num = reg_stats.Area;
    imrv = reshape(rgb_reg_nan(:,:,1),1,len);
    imgv = reshape(rgb_reg_nan(:,:,2),1,len);
    imbv = reshape(rgb_reg_nan(:,:,3),1,len);
    imiv = 0.2989*imrv+0.5870*imgv+0.1140*imbv;

    % set up bins
    binl = (max_rgb-min_rgb)/cbins;
    binv = linspace(min_rgb,max_rgb,cbins+1);
    bint = linspace(min_rgb+binl/2,max_rgb-binl/2,cbins);

    % histogram (normalize by number of pixels in region)
    rdis = histc(imrv,binv)./num;
    gdis = histc(imgv,binv)./num;
    bdis = histc(imbv,binv)./num;
    idis = histc(imiv,binv)./num;

    toc

    % plot
    if (figures)
        figure; 
        subplot(4,1,1); bar(bint,rdis(1:cbins));
        title('Red intensity (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
        subplot(4,1,2); bar(bint,gdis(1:cbins));
        title('Green intensity (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
        subplot(4,1,3); bar(bint,bdis(1:cbins));
        title('Blue intensity (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
        subplot(4,1,4); bar(bint,idis(1:cbins));
        title('Grayscale intensity (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
    end

    %% histogram colors and intensity (superpixels)
    if (messages)
        fprintf('Computing superpixel histograms of colors ...\n');
    end

    tic

    % apply median filter
    imr = medfilt2(rgb_reg_nan(:,:,1), [sp_size sp_size]);
    img = medfilt2(rgb_reg_nan(:,:,2), [sp_size sp_size]);
    imb = medfilt2(rgb_reg_nan(:,:,3), [sp_size sp_size]);

    % build the superpixel matrices
    srows = round(rows/sp_size);
    scols = round(cols/sp_size);
    slen = srows*scols;
    simr = zeros(srows,scols,'single');
    simg = zeros(srows,scols,'single');
    simb = zeros(srows,scols,'single');
    sreg_mask = false(srows,scols);
    ii = ceil(sp_size/2);
    for i=1:srows
        jj = ceil(sp_size/2);
        for j=1:scols
            simr(i,j) = imr(ii,jj);
            simg(i,j) = img(ii,jj);
            simb(i,j) = imb(ii,jj);
            if (~isnan(simr(i,j)))
                sreg_mask(i,j) = true;
            end
            jj = jj+sp_size;
        end
        ii = ii+sp_size;
    end

    % count new area
    snum = sum(sum(sreg_mask));

    % back into vectors
    simrv = reshape(simr,1,slen);
    simgv = reshape(simg,1,slen);
    simbv = reshape(simb,1,slen);
    simiv = 0.2989*simrv+0.5870*simgv+0.1140*simbv;

    % set up bins
    binl = (max_rgb-min_rgb)/scbins;
    binv = linspace(min_rgb,max_rgb,scbins+1);
    bint = linspace(min_rgb+binl/2,max_rgb-binl/2,scbins);

    % histogram (normalize by number of pixels in region)
    srdis = histc(simrv,binv)./snum;
    sgdis = histc(simgv,binv)./snum;
    sbdis = histc(simbv,binv)./snum;
    sidis = histc(simiv,binv)./snum;

    toc

    % plot
    if (figures)
        spimg = zeros(srows,scols,3,'uint8');
        spimg(:,:,1) = uint8(simr);
        spimg(:,:,2) = uint8(simg);
        spimg(:,:,3) = uint8(simb);
        figure; imshow(spimg);
        title(['Regions collection ', num2str(reg), ' (superpixels)']);
        figure; 
        subplot(4,1,1); bar(bint,srdis(1:scbins));
        title('Red intensity (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
        subplot(4,1,2); bar(bint,sgdis(1:scbins));
        title('Green intensity (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
        subplot(4,1,3); bar(bint,sbdis(1:scbins));
        title('Blue intensity (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
        subplot(4,1,4); bar(bint,sidis(1:scbins));
        title('Grayscale intensity (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_rgb max_rgb 0 1]); set(gca,'xtick', binv);
    end

    % %% histogram textons map (full resolution) 
    if (messages)
        fprintf('Computing pixel histogram of textons ...\n');
    end

    tic

    % reshape
    tmv=single(reshape(text_reg_nan,1,len));

    % set up bins
    binl = (max_text-min_text)/tmbins;
    binv = linspace(min_text,max_text,tmbins+1);
    bint = linspace(min_text+binl/2,max_text-binl/2,tmbins);

    % histogram (normalize by number of pixels in region)
    tmdis = histc(tmv,binv)./num;

    toc

    % plot
    if (figures)
        figure; imshow(uint8(reshape(tmv, rows, cols)));
        title('Texton map (pixels)');
        figure; bar(bint,tmdis(1:tmbins));
        title('Texton map (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_text max_text 0 1]); set(gca,'xtick', binv);
    end

    %% histogram range filter (full resolution) 
    if (messages)
        fprintf('Computing pixel histogram of intensity range ...\n');
    end

    tic

    % apply range filter
    range = ordfilt2(gray_reg_nan, rf_size*rf_size, true(rf_size)) - ordfilt2(gray_reg_nan, 1, true(rf_size));
    rfv=single(reshape(range,1,len));

    % note: the above operation erodes the region, must re-count pixels!
    rnum = 0;
    for i=1:len
        if (~isnan(rfv(i)))
                rnum = rnum+1;
        end
    end

    % set up bins
    binl = (max_gray-min_gray)/rfbins;
    binv = linspace(min_gray,max_gray,rfbins+1);
    bint = linspace(min_gray+binl/2,max_gray-binl/2,rfbins);

    % histogram (normalize by number of pixels in region)
    rfdis = histc(rfv,binv)./rnum;

    toc

    % plot
    if (figures)
        figure; imshow(uint8(reshape(rfv, rows, cols)));
        title('Range filter (pixels)');
        figure; bar(bint,rfdis(1:rfbins));
        title('Range filter (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_gray max_gray 0 1]); set(gca,'xtick', binv);
    end

    %% histogram range filter (superpixels)
    if (messages)
        fprintf('Computing superpixel histogram of intensity range ...\n');
    end
    % note: could result in zero distribution if particle is too small

    tic

    % apply median filter
    medim = medfilt2(gray_reg_nan, [sp_size sp_size]);

    % build the superpixel matrix
    sgray_reg_nan = zeros(srows,scols,'single');
    ii = ceil(sp_size/2);
    for i=1:srows
        jj = ceil(sp_size/2);
        for j=1:scols
            sgray_reg_nan(i,j) = medim(ii,jj);
            jj = jj+sp_size;
        end
        ii = ii+sp_size;
    end

    % apply range filter
    srange = ordfilt2(sgray_reg_nan, rf_size*rf_size, true(rf_size)) - ordfilt2(sgray_reg_nan, 1, true(rf_size));
    srfv = single(reshape(srange,1,slen));

    % note: the above operation erodes the region, must re-count pixels!
    srnum = 0;
    for i=1:slen
        if (~isnan(srfv(i)))
                srnum = srnum+1;
        end
    end

    % set up bins
    binl = (max_gray-min_gray)/srfbins;
    binv = linspace(min_gray,max_gray,srfbins+1);
    bint = linspace(min_gray+binl/2,max_gray-binl/2,srfbins);

    % histogram (normalize by number of pixels in region)
    srfdis = histc(srfv,binv)./srnum;

    toc

    % plot
    if (figures)
        figure; imshow(uint8(reshape(srfv, srows, scols)));
        title('Range filter (super pixels)');
        figure; bar(bint,srfdis(1:srfbins));
        title('Range filter (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_gray max_gray 0 1]); set(gca,'xtick', binv);
    end

    %% histogram entropy filter (full resolution)
    % note: entropyfilt cannot deal with NaN's and cannot be modified (mexfile).
    % the filter is thus applied to the whole image, and masked with region.
    % to avoid boundary effects the mask is eroded with the same filter size.

    if (messages)
        fprintf('Computing pixel histogram of neighborhood entropy ...\n');
    end

    tic

    % erode mask using filter size and count pixels
    emask = imerode(reg_mask, true(ef_size));
    enum = sum(sum(emask));

    % apply entropy filter to whole image
    % note: filtering could be done once and passed to the function for efficiency
    entr_reg = double(entropyfilt(gray_img, true(ef_size)));

    % mask and reshape
    for i=1:rows
        for j=1:cols
            if (~emask(i,j))
                entr_reg(i,j) = NaN;
            end
        end
    end
    efv=single(reshape(entr_reg,1,len));

    % set up bins
    binl = (max_entr-min_entr)/efbins;
    binv = linspace(min_entr,max_entr,efbins+1);
    bint = linspace(min_entr+binl/2,max_entr-binl/2,efbins);

    % histogram (normalize by number of pixels in region)
    efdis = histc(efv,binv)./enum;

    toc

    % plot
    if (figures)
        figure; bar(bint,efdis(1:efbins));
        title('Entropy filter (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_entr max_entr 0 1]); set(gca,'xtick', binv);
    end

    %% histogram entropy filter (superpixels)
    % note: entropyfilt cannot deal with NaN's and cannot be modified (mexfile).
    % the filter is thus applied to the whole image, and masked with region.
    % to avoid boundary effects the mask is eroded with the same filter size.

    if (messages)
        fprintf('Computing superpixel histogram of neighborhood entropy ...\n');
    end

    tic

    % note: superpixel gray image could be computed once and passed to function
    % apply median filter
    medim = medfilt2(gray_img, [sp_size sp_size]);

    % build the superpixel matrix
    sgray_img = zeros(srows,scols,'uint8');
    ii = ceil(sp_size/2);
    for i=1:srows
        jj = ceil(sp_size/2);
        for j=1:scols
            sgray_img(i,j) = medim(ii,jj);
            jj = jj+sp_size;
        end
        ii = ii+sp_size;
    end

    % erode mask using filter size and count pixels
    semask = imerode(sreg_mask, true(ef_size));
    senum = sum(sum(semask));

    % apply entropy filter to whole image
    % note: filtering could be done once and passed to the function for efficiency
    sentr_reg = double(entropyfilt(sgray_img, true(ef_size)));

    % mask and reshape
    for i=1:srows
        for j=1:scols
            if (~semask(i,j))
                sentr_reg(i,j) = NaN;
            end
        end
    end
    sefv=single(reshape(sentr_reg,1,slen));

    % set up bins
    binl = (max_entr-min_entr)/sefbins;
    binv = linspace(min_entr,max_entr,sefbins+1);
    bint = linspace(min_entr+binl/2,max_entr-binl/2,sefbins);

    % histogram (normalize by number of pixels in region)
    sefdis = histc(sefv,binv)./senum;

    toc

    % plot
    if (figures)
        figure; bar(bint,sefdis(1:sefbins));
        title('Entropy filter (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([min_entr max_entr 0 1]); set(gca,'xtick', binv);
    end

    %% Gray-Level Co-Occurrence Matrix Correlation (full resolution)
    if (messages)
        fprintf('Computing co-occurrence matrix correlation ...\n');
    end

    tic

    % create offsets vector
    offsets = [ ...
        zeros(cm_size,1) (1:cm_size)'; ...  % horizontal (right)
        (1:cm_size)' zeros(cm_size,1) ; ... % vertical (down)
        (1:cm_size)' (1:cm_size)' ; ...     % diagonal (down-right)
        ];

    % compute matrix
    warning('off', 'Images:graycomatrix:scaledImageContainsNan');
    glcms = graycomatrix(gray_reg_nan, 'Offset', offsets, 'GrayLimits', [0 255], 'NumLevels', 256);

    % compute correlation
    cor_stats = graycoprops(glcms, 'Correlation');

    % set up bins
    binl = 2/ccbins;
    binv = linspace(-1,1,ccbins+1);
    bint = linspace(-1+binl/2,1-binl/2,ccbins);

    % histogram (normalize by number of offsets?)
    ccdish = histc(cor_stats.Correlation(1:cm_size),binv)./cm_size;
    ccdisv = histc(cor_stats.Correlation(cm_size+1:2*cm_size),binv)./cm_size;
    ccdisd = histc(cor_stats.Correlation((2*cm_size)+1:3*cm_size),binv)./cm_size;

    toc

    % plot
    if (figures)
        figure; 
        subplot(3,1,1); title('Intensity correlation as a function of offset');
        plot([cor_stats.Correlation(1:cm_size)]);
        xlabel('Horizontal Offset'); ylabel('Correlation');
        subplot(3,1,2); plot([cor_stats.Correlation(cm_size+1:2*cm_size)]);
        xlabel('Vertical Offset'); ylabel('Correlation');
        subplot(3,1,3); plot([cor_stats.Correlation((2*cm_size)+1:3*cm_size)]);
        xlabel('Diagonal Offset'); ylabel('Correlation');
        figure; 
        subplot(3,1,1); bar(bint,ccdish(1:ccbins));
        title('Horizontal correlation (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([-1 1 0 1]); set(gca,'xtick', binv);
        subplot(3,1,2); bar(bint,ccdisv(1:ccbins));
        title('Vertical correlation (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([-1 1 0 1]); set(gca,'xtick', binv);
        subplot(3,1,3); bar(bint,ccdisd(1:ccbins));
        title('Diagonal correlation (pixels)'); xlabel('bin'); ylabel('% in bin');
        axis([-1 1 0 1]); set(gca,'xtick', binv);
    end

    %% Gray-Level Co-Occurrence Matrix Correlation (superpixels)
    if (messages)
        fprintf('Computing co-occurrence matrix correlation ...\n');
    end

    tic

    % create offsets vector
    soffsets = [ ...
        zeros(scm_size,1) (1:scm_size)'; ...  % horizontal (right)
        (1:scm_size)' zeros(scm_size,1) ; ... % vertical (down)
        (1:scm_size)' (1:scm_size)' ; ...     % diagonal (down-right)
        ];

    % compute matrix
    sglcms = graycomatrix(simg, 'Offset', soffsets, 'GrayLimits', [0 255], 'NumLevels', 256);

    % compute correlation
    scor_stats = graycoprops(sglcms, 'Correlation');

    % set up bins
    binl = 2/sccbins;
    binv = linspace(-1,1,sccbins+1);
    bint = linspace(-1+binl/2,1-binl/2,sccbins);

    % histogram (normalize by number of offsets?)
    sccdish = histc(scor_stats.Correlation(1:scm_size),binv)./scm_size;
    sccdisv = histc(scor_stats.Correlation(scm_size+1:2*scm_size),binv)./scm_size;
    sccdisd = histc(scor_stats.Correlation((2*scm_size)+1:3*scm_size),binv)./scm_size;

    toc

    % plot
    if (figures)
        figure; 
        subplot(3,1,1); title('Intensity correlation as a function of offset (superpixels)');
        plot([scor_stats.Correlation(1:scm_size)]);
        xlabel('Horizontal Offset'); ylabel('Correlation');
        subplot(3,1,2); plot([scor_stats.Correlation(scm_size+1:2*scm_size)]);
        xlabel('Vertical Offset'); ylabel('Correlation');
        subplot(3,1,3); plot([scor_stats.Correlation((2*scm_size)+1:3*scm_size)]);
        xlabel('Diagonal Offset'); ylabel('Correlation');
        figure; 
        subplot(3,1,1); bar(bint,sccdish(1:sccbins));
        title('Horizontal correlation (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([-1 1 0 1]); set(gca,'xtick', binv);
        subplot(3,1,2); bar(bint,sccdisv(1:sccbins));
        title('Vertical correlation (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([-1 1 0 1]); set(gca,'xtick', binv);
        subplot(3,1,3); bar(bint,sccdisd(1:sccbins));
        title('Diagonal correlation (superpixels)'); xlabel('bin'); ylabel('% in bin');
        axis([-1 1 0 1]); set(gca,'xtick', binv);
    end

    %% grayscale entropy value (full resolution)
    if (messages)
        fprintf('Computing global grayscale entropy ...\n');
    end

    tic

    % set up bins
    binv = linspace(min_gray,max_gray,egbins+1);

    % histogram grayscale image (normalize by number of pixels in region)
    gsdis = histc(reshape(gray_reg_nan,1,len),binv)./num;

    % remove zero entries from distribution
    gsdis(gsdis==0) = [];

    % compute entropy
    gentr_reg = -sum(gsdis.*log2(gsdis));

    toc

    %% grayscale entropy value (superpixels)
    if (messages)
        fprintf('Computing global grayscale entropy (superpixels) ...\n');
    end

    tic

    % histogram grayscale image (normalize by number of pixels in region)
    sgsdis = histc(reshape(sgray_reg_nan,1,slen),binv)./snum;

    % remove zero entries from distribution
    sgsdis(sgsdis==0) = [];

    % compute entropy
    sgentr_reg = -sum(sgsdis.*log2(sgsdis));

    toc

    %% create descriptor
    descriptor = [];
    if (selection(1))
        descriptor = [descriptor reg_stats.Area];
    end
    if (selection(2))
        descriptor = [descriptor reg_stats.Perimeter];
    end
    if (selection(3))
        descriptor = [descriptor reg_stats.Eccentricity];
    end
    if (selection(4))
        descriptor = [descriptor reg_stats.Solidity];
    end
    if (selection(5))
        descriptor = [descriptor circularity];
    end
    if (selection(6))
        descriptor = [descriptor odis(1:obins)];
    end
    if (selection(7))
        descriptor = [descriptor kdis(1:kbins)];
    end
    if (selection(8))
        descriptor = [descriptor qdis(1:qbins)];
    end
    if (selection(9))
        descriptor = [descriptor rdis(1:cbins) gdis(1:cbins) bdis(1:cbins)];
    end
    if (selection(10))
        descriptor = [descriptor srdis(1:scbins) sgdis(1:scbins) sbdis(1:scbins)];
    end
    if (selection(11))
        descriptor = [descriptor idis(1:cbins)];
    end
    if (selection(12))
        descriptor = [descriptor sidis(1:scbins)];
    end
    if (selection(13))
        descriptor = [descriptor tmdis(1:tmbins)];
    end
    if (selection(14))
        descriptor = [descriptor rfdis(1:rfbins)];
    end
    if (selection(15))
        descriptor = [descriptor srfdis(1:srfbins)];
    end
    if (selection(16))
        descriptor = [descriptor efdis(1:efbins)];
    end
    if (selection(17))
        descriptor = [descriptor sefdis(1:sefbins)];
    end
    if (selection(18))
        descriptor = [descriptor ...
            cor_stats.Correlation(1:cm_size) ...
            cor_stats.Correlation(cm_size+1:2*cm_size) ...
            cor_stats.Correlation((2*cm_size)+1:3*cm_size) ...
            ];
    end
    if (selection(19))
        descriptor = [descriptor ...
            scor_stats.Correlation(1:scm_size) ...
            scor_stats.Correlation(scm_size+1:2*scm_size) ...
            scor_stats.Correlation((2*scm_size)+1:3*scm_size) ...
            ];
    end
    if (selection(20))
        descriptor = [descriptor gentr_reg];
    end
    if (selection(21))
        descriptor = [descriptor sgentr_reg];
    end

    if (messages)
        fprintf(['Descriptor size: ', num2str(size(descriptor,2)), '\n']);
    end

    if (saves)
        save([baseName '_desc_' num2str(reg)], 'descriptor');
    end
end