function [svmModel, svmType, svmParams, svmOpts, labels, accuracy, value] = ...
    trainSVM(baseName, labels, params, options)
% author: dino bellugi
% copyright: blueG soft wear 2010
% trains SVM given training data (libSVM version)
% inputs:
%   baseName = base name for files
%   labels = row vector with labels correspomding to the descriptors
%   params = parameters for training
%   params(1): 0=load model, 1=find best parameters, 2=use defaults
%   params(2): 0=linear, 1=poly, 2=rbf, 3=sigmoid
%   params(3): maximum degree (for poly only)
%   params(4): number of subsets for cross-validation
%   params(5): lower bound of search space (log)
%   params(6): upper bound of search space (log)
%   options = logical(1,3) for print, save, test
% outputs:
%   will come soon
% example:
%   [model, type, pars, opts, lab, acc, val] = ...
%       trainSVM('../newData/Img0057_cropped', ...
%       labels, [1 1 10 5 -5 5], [true true true]);
% notes:
%   has some hard-coded parameters (see below)
%   expects libSVM in path:
%   addpath('../libSVM')
%   addpath('../libSVM/libsvm-mat-2.89-3')
% naming convention:
%   descriptorsName = [baseName '_descriptors']
%   modelName = [baseName '_model']

    %% check command line
    if (nargin < 3 || nargin > 4)
        error(['Usage: [svmModel, svmType, svmParams, svmOpts, labels, ', ...
            'accuracy, value] = trainSVM(baseName, labels, params, [options])\n']);
    end
    if (nargin < 4)
        options = true(3,1);
    end
    
    %% setup naming structure
    descriptorsName = [baseName '_descriptors'];
    modelName = [baseName '_model'];

    %% load and check data
    load(descriptorsName);
    numDesc = size(descriptors, 2);

    if (numDesc ~= size(labels, 2))
        error('trainSVM error: labels and descriptors do not match!\n');
    end
    
    trainData = descriptors';
    trainLabels = labels';

    % brutally replace NaN with 0's
    trainData(isnan(trainData)) = 0;

    % type, options, automate
    svm_mode = params(1);	% 0 = load, 1 = search, 2 = default
    ker_type = params(2);   % 0 = linear, 1 = poly, etc.
    max_deg = params(3);    % max degree of poly
    sub_num = params(4);    % number of subsets for cross validation
    min_search = params(5); % lower bound of search space (log)
    max_search = params(6); % upper bound of search space (log)
        
    % define default parameters
    % -s svm_type : set type of SVM (default 0)
    %         0 -- C-SVC
    %         1 -- nu-SVC
    %         2 -- one-class SVM
    %         3 -- epsilon-SVR
    %         4 -- nu-SVR
    % -t kernel_type : set type of kernel function (default 2)
    %         0 -- linear: u'*v
    %         1 -- polynomial: (gamma*u'*v + coef0)^degree
    %         2 -- radial basis function: exp(-gamma*|u-v|^2)
    %         3 -- sigmoid: tanh(gamma*u'*v + coef0)
    %         4 -- precomputed kernel (kernel values in training_set_file)
    % -d degree : set degree in kernel function (default 3)
    % -g gamma : set gamma in kernel function (default 1/k)
    % -r coef0 : set coef0 in kernel function (default 0)
    % -c cost : set the parameter C of C-SVC, epsilon-SVR, 
    %   and nu-SVR (default 1)
    % -n nu : set the parameter nu of nu-SVC, one-class SVM, 
    %   and nu-SVR (default 0.5)
    % -p epsilon : set the epsilon in loss function of epsilon-SVR (default 0.1)
    % -m cachesize : set cache memory size in MB (default 100)
    % -e epsilon : set tolerance of termination criterion (default 0.001)
    % -h shrinking : whether to use the shrinking heuristics, 0 or 1 (default 1)
    % -b probability_estimates : whether to train a SVC or SVR 
    %   model for probability estimates, 0 or 1 (default 0)
    % -wi weight : set the parameter C of class i to weight*C, 
    %   for C-SVC (default 1)
    % -v n: n-fold cross validation mode
    % -q : quiet mode (no outputs)

    ker_opts = ' -d 5';
    svm_type = [' -t ', num2str(ker_type), ' ', ker_opts];   % svm type, etc.
    svm_opts = ' -b 1 ';              % quiet mode, etc.

    %% get model parameters
    % set cost and gamma parameters
    if (svm_mode == 0)
        if (options(1))
            fprintf(['Loading SVM model ', modelName, ' ...\n']);
        end
        load(modelName, 'svm_model', 'svm_type', 'svm_params', 'svm_opts');   
    elseif (svm_mode == 1)
        % find best parameters using cross-correlation
        if (options(1))
            fprintf('Searching for best SVM parameters ...\n');
        end
        tic
        bestcv = 0;
        if (ker_type == 1)
            for deg = 1:max_deg
                ker_opts = [' -d ', num2str(deg), ' '];
                for log2c = min_search:max_search;
                    for log2g = min_search:max_search;
                        svm_cmd = ['-v ', num2str(sub_num), ' -c ', ...
                            num2str(2^log2c), ' -g ', num2str(2^log2g)];
                        cv = svmtrain(trainLabels, trainData, ...
                            [svm_cmd svm_type ker_opts]);
                        if (cv > bestcv),
                            bestdeg = deg; bestcv = cv; 
                            bestc = 2^log2c; bestg = 2^log2g;
                        end
                        if (options(1))
                            fprintf(['deg=%g ln(c)=%g ln(g)=%g x-val=%g ', ...
                                '(best deg=%g, best c=%g, g=%g, rate=%g)\n'], ...
                                deg, log2c, log2g, cv, bestdeg, bestc, bestg, ...
                                bestcv);
                        end
                    end
                end
            end
            ker_opts = [' -d ', num2str(bestdeg), ' '];
            svm_params = [ker_opts, ' -c ', num2str(bestc), ' -g ', ...
                num2str(bestg)];
         else
            for log2c = min_search:max_search;
                for log2g = min_search:max_search;
                    svm_cmd = ['-v ', num2str(sub_num), ' -c ', ...
                        num2str(2^log2c), ' -g ', num2str(2^log2g)];
                    cv = svmtrain(trainLabels, trainData, ...
                        [svm_cmd svm_type]);
                    if (cv > bestcv),
                        bestcv = cv; bestc = 2^log2c; bestg = 2^log2g;
                    end
                    if (options(1))
                        fprintf(['ln(c)=%g ln(g)=%g x-val=%g ', ...
                            '(best c=%g, g=%g, rate=%g)\n'], ...
                            log2c, log2g, cv, bestc, bestg, bestcv);
                    end
                end
            end
            svm_params = ['-c ', num2str(bestc), ' -g ', num2str(bestg)];
        end
        toc
    else
        if (options(1))
            fprintf('Using default SVM parameters ...\n');
        end
        % define here or leave blank for defaults
        svm_params = '';
    end

    %% train
    if (options(1))
        fprintf('Training SVM ...\n');
    end
    tic
    svm_model = svmtrain(trainLabels, trainData, ...
        [svm_type svm_params svm_opts]);
    toc

    %% save
    if (options(2))
        if (options(1))
            fprintf('Saving SVM ...\n');
        end
        save(modelName, 'svm_model', 'svm_type', 'svm_params', 'svm_opts');
    end

    %% test
    if (options(3))
        if (options(1))
            fprintf('Testing SVM ...\n');
        end
        tic
        [labels, accuracy, value] = svmpredict(trainLabels, ...
            trainData, svm_model, svm_opts);
        toc
    end

end